/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */

#include "rb-papi.h"

static VALUE cAttributeList;

static void
no_free(void *ptr)
{
}

VALUE
rb_papi_c2r_attribute_list(papi_attribute_t **attrs)
{
    return Data_Wrap_Struct(cAttributeList, 0, papiAttributeListFree, attrs);
}

VALUE
rb_papi_c2r_attribute_list_no_free(papi_attribute_t **attrs)
{
    return Data_Wrap_Struct(cAttributeList, 0, no_free, attrs);
}

static VALUE
rb_papi_attribute_list_allocate(VALUE klass)
{
    return Data_Wrap_Struct(klass, NULL, papiAttributeListFree, NULL);
}

static VALUE
rb_papi_attribute_list_initialize(int argc, VALUE *argv, VALUE self)
{
    VALUE buffer, rb_attrs, rb_add_flags;
    papi_attribute_t **attrs = NULL;

    rb_scan_args(argc, argv, "03", &buffer, &rb_attrs, &rb_add_flags);

    if (!NIL_P(buffer)) {
        papi_status_t status;
        int add_flags = PAPI_ATTR_EXCL;

        if (!NIL_P(rb_attrs)) {
            attrs = RVAL2ATTRLST(rb_attrs);
            if (!NIL_P(rb_add_flags))
                add_flags = NUM2INT(rb_add_flags);
        }

        status = papiAttributeListFromString(&attrs, add_flags,
                                             StringValueCStr(buffer));
        rb_papi_status_check(status);
    }

    RDATA(self)->data = attrs;

    return Qnil;
}

static VALUE
rb_papi_attribute_list_add(int argc, VALUE *argv, VALUE self)
{
    VALUE name, value, flags;
    papi_attribute_t *attr;
    papi_attribute_t ***attrs;
    papi_attribute_t **dummy_attrs;

    rb_scan_args(argc, argv, "21", &name, &value, &flags);

    attrs = (papi_attribute_t ***)&(RDATA(self)->data);
    attr = rb_papi_r2c_attribute(name, value);
    rb_papi_status_check(papiAttributeListAddValue(attrs,
                                                   NIL_P(flags) ?
                                                     PAPI_ATTR_APPEND :
                                                     NUM2INT(flags),
                                                   attr->name,
                                                   attr->type,
                                                   *(attr->values)));

    dummy_attrs = ALLOC_N(papi_attribute_t *, 2);
    dummy_attrs[0] = attr;
    dummy_attrs[1] = NULL;
    papiAttributeListFree(dummy_attrs);

    return self;
}

static VALUE
rb_papi_attribute_list_find(VALUE self, VALUE name)
{
    VALUE result = Qnil;
    papi_attribute_t **attrs = RVAL2ATTRLST(self);
    papi_attribute_t *attr;

    if (attrs) {
        attr = papiAttributeListFind(attrs, StringValueCStr(name));
        if (attr) {
            result = ATTR2RVAL_NO_FREE(attr);
        }
    }

    return result;
}

static VALUE
rb_papi_attribute_list_each(VALUE self)
{
    VALUE rb_attrs = rb_ary_new();
    papi_attribute_t **attrs = RVAL2ATTRLST(self);
    papi_attribute_t *attr;
    void *iter = NULL;

    if (attrs) {
        while ((attr = papiAttributeListGetNext(attrs, &iter))) {
            VALUE rb_attr = ATTR2RVAL_NO_FREE(attr);
            rb_ary_push(rb_attrs, rb_attr);
            rb_yield(rb_attr);
        }
    }

    return rb_attrs;
}

static VALUE
rb_papi_attribute_list_to_string(int argc, VALUE *argv, VALUE self)
{
    VALUE rb_delim, result;
    char *delim, *buffer;
    papi_attribute_t **attrs;
    papi_status_t status = PAPI_NOT_POSSIBLE;
    int i;

    rb_scan_args(argc, argv, "01", &rb_delim);

    attrs = RVAL2ATTRLST(self);
    delim = NIL_P(rb_delim) ? NULL : StringValuePtr(rb_delim);

    buffer = ALLOC_N(char, BUFFER_SIZE);
    status = papiAttributeListToString(attrs, delim, buffer, BUFFER_SIZE);
    for (i = 2; status != PAPI_OK; i++) {
        if (i > 10) {
            free(buffer);
            rb_raise(rb_eNoMemError, "%s", papiStatusString(status));
        }
        REALLOC_N(buffer, char, BUFFER_SIZE * i);
        status = papiAttributeListToString(attrs, delim,
                                           buffer, BUFFER_SIZE * i);
    }
    result = rb_str_new2(buffer);
    free(buffer);

    return result;
}


void
Init_rb_papi_attribute_list(VALUE mPAPI)
{
    cAttributeList = rb_define_class_under(mPAPI, "AttributeList", rb_cData);

    rb_include_module(cAttributeList, rb_mEnumerable);
    rb_define_alloc_func(cAttributeList, rb_papi_attribute_list_allocate);
    rb_define_method(cAttributeList, "initialize",
                     rb_papi_attribute_list_initialize, -1);

    rb_define_method(cAttributeList, "find", rb_papi_attribute_list_find, 1);
    rb_define_method(cAttributeList, "each", rb_papi_attribute_list_each, 0);
    rb_define_method(cAttributeList, "to_s",
                     rb_papi_attribute_list_to_string, -1);

    rb_define_method(cAttributeList, "add",
                     rb_papi_attribute_list_add, -1);
}
