/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */

#include "rb-papi.h"

static ID id_new;
static VALUE cFilter, cFilterBitmask;


VALUE
rb_papi_c2r_filter(papi_filter_t *filter)
{
    papi_filter_t *ptr = ALLOC(papi_filter_t);
    ptr = filter;
    return Data_Wrap_Struct(cFilter, 0, 0, ptr);
}

static VALUE
rb_papi_filter_get_type(VALUE self)
{
    return INT2NUM(RVAL2FILTER(self)->type);
}

static VALUE
rb_papi_filter_get_filter(VALUE self)
{
    VALUE result = Qnil;
    papi_filter_t *filter;

    filter = RVAL2FILTER(self);

    switch (filter->type) {
      case PAPI_FILTER_BITMASK:
        result = rb_funcall(cFilterBitmask,
                            id_new,
                            2,
                            UINT2NUM(filter->filter.bitmask.mask),
                            UINT2NUM(filter->filter.bitmask.value));
        break;
      default:
        rb_raise(rb_eArgError, "invalid filter type %d", filter->type);
        break;
    }

    return result;
}


void
Init_rb_papi_filter(VALUE mPAPI)
{
    VALUE mFilterType;

    id_new = rb_intern("new");

    mFilterType = rb_define_module_under(mPAPI, "FilterType");
    cFilter = rb_define_class_under(mPAPI, "Filter", rb_cData);
    cFilterBitmask = rb_struct_define("FilterBitmask", "mask", "value", NULL);
    rb_define_const(mPAPI, "FilterBitmask", cFilterBitmask);

    /* papi_filter_type_t */
    rb_define_const(mFilterType, "BITMASK", INT2NUM(PAPI_FILTER_BITMASK));

    /* papi_filter_t */
    rb_define_method(cFilter, "type", rb_papi_filter_get_type, 0);
    rb_define_method(cFilter, "filter", rb_papi_filter_get_filter, 0);

    /* PAPI_PRINTER_* for bitmask filter */
    rb_define_const(cFilterBitmask, "LOCAL", INT2NUM(PAPI_PRINTER_LOCAL));
    rb_define_const(cFilterBitmask, "CLASS", INT2NUM(PAPI_PRINTER_CLASS));
    rb_define_const(cFilterBitmask, "REMOTE", INT2NUM(PAPI_PRINTER_REMOTE));
    rb_define_const(cFilterBitmask, "BW", INT2NUM(PAPI_PRINTER_BW));
    rb_define_const(cFilterBitmask, "COLOR", INT2NUM(PAPI_PRINTER_COLOR));
    rb_define_const(cFilterBitmask, "DUPLEX", INT2NUM(PAPI_PRINTER_DUPLEX));
    rb_define_const(cFilterBitmask, "STAPLE", INT2NUM(PAPI_PRINTER_STAPLE));
    rb_define_const(cFilterBitmask, "COPIES", INT2NUM(PAPI_PRINTER_COPIES));
    rb_define_const(cFilterBitmask, "COLLATE", INT2NUM(PAPI_PRINTER_COLLATE));
    rb_define_const(cFilterBitmask, "PUNCH", INT2NUM(PAPI_PRINTER_PUNCH));
    rb_define_const(cFilterBitmask, "COVER", INT2NUM(PAPI_PRINTER_COVER));
    rb_define_const(cFilterBitmask, "BIND", INT2NUM(PAPI_PRINTER_BIND));
    rb_define_const(cFilterBitmask, "SORT", INT2NUM(PAPI_PRINTER_SORT));
    rb_define_const(cFilterBitmask, "SMALL", INT2NUM(PAPI_PRINTER_SMALL));
    rb_define_const(cFilterBitmask, "MEDIUM", INT2NUM(PAPI_PRINTER_MEDIUM));
    rb_define_const(cFilterBitmask, "LARGE", INT2NUM(PAPI_PRINTER_LARGE));
    rb_define_const(cFilterBitmask, "VARIABLE", INT2NUM(PAPI_PRINTER_VARIABLE));
    rb_define_const(cFilterBitmask, "IMPLICIT", INT2NUM(PAPI_PRINTER_IMPLICIT));
    rb_define_const(cFilterBitmask, "DEFAULT", INT2NUM(PAPI_PRINTER_DEFAULT));
    rb_define_const(cFilterBitmask, "OPTIONS", INT2NUM(PAPI_PRINTER_OPTIONS));
}
