/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */

#include "rb-papi.h"

static ID id_service, id_call;
static VALUE cJob, cJobTicket, cJobStream;

static void
no_free(void *ptr)
{
}

VALUE
rb_papi_c2r_job(papi_job_t job)
{
    return Data_Wrap_Struct(cJob, 0, papiJobFree, job);
}

VALUE
rb_papi_c2r_job_no_free(papi_job_t job)
{
    return Data_Wrap_Struct(cJob, 0, no_free, job);
}

VALUE
rb_papi_c2r_job_ticket(papi_job_ticket_t *job_ticket)
{
    return Data_Wrap_Struct(cJobTicket, 0, 0, job_ticket);
}

VALUE
rb_papi_c2r_job_ticket_no_free(papi_job_ticket_t *job_ticket)
{
    return Data_Wrap_Struct(cJobTicket, 0, no_free, job_ticket);
}

VALUE
rb_papi_c2r_job_stream_no_free(papi_stream_t stream)
{
    return Data_Wrap_Struct(cJobStream, 0, no_free, stream);
}

/* JobTicket related */
static VALUE
rb_papi_job_ticket_allocate(VALUE klass)
{
    return Data_Wrap_Struct(klass, NULL, NULL, NULL);
}

static VALUE
rb_papi_job_ticket_initialize(VALUE self, VALUE format,
                              VALUE data, VALUE file_name)
{
    papi_job_ticket_t *ticket;

    ticket = ALLOC(papi_job_ticket_t);

    ticket->format = NUM2INT(format);
    ticket->ticket_data = StringValueCStr(data);
    ticket->file_name = StringValueCStr(file_name);

    RDATA(self)->data = ticket;

    return Qnil;
}

static VALUE
rb_papi_job_ticket_get_format(VALUE self)
{
    return INT2NUM(RVAL2JT(self)->format);
}

static VALUE
rb_papi_job_ticket_get_data(VALUE self)
{
    return rb_str_new2(RVAL2JT(self)->ticket_data);
}

static VALUE
rb_papi_job_ticket_get_file_name(VALUE self)
{
    return rb_str_new2(RVAL2JT(self)->file_name);
}

static VALUE
rb_papi_job_ticket_set_format(VALUE self, VALUE format)
{
    RVAL2JT(self)->format = NUM2INT(format);
    return Qnil;
}

static VALUE
rb_papi_job_ticket_set_data(VALUE self, VALUE data)
{
    RVAL2JT(self)->ticket_data = StringValueCStr(data);
    return Qnil;
}

static VALUE
rb_papi_job_ticket_set_file_name(VALUE self, VALUE file_name)
{
    RVAL2JT(self)->file_name = StringValueCStr(file_name);
    return Qnil;
}



/* Job related */
static VALUE
rb_papi_job_submit(int argc, VALUE *argv, VALUE self)
{
    VALUE service, printer, job_attributes, job_ticket, rb_files;
    VALUE rb_job;
    papi_job_t job;
    char **files;

    rb_scan_args(argc, argv, "23", &service, &printer, &job_attributes,
                 &job_ticket, &rb_files);

    ARY2STRARY(rb_files, files);
    rb_papi_status_check(papiJobSubmit(RVAL2SERVICE(service),
                                       StringValueCStr(printer),
                                       RVAL2ATTRLST(job_attributes),
                                       RVAL2JT(job_ticket),
                                       files,
                                       &job));
    rb_job = JOB2RVAL(job);
    rb_ivar_set(rb_job, id_service, service);
    return rb_job;
}

static VALUE
rb_papi_job_submit_by_reference(int argc, VALUE *argv, VALUE self)
{
    VALUE service, printer, job_attributes, job_ticket, rb_files;
    VALUE rb_job;
    papi_job_t job;
    char **files;

    rb_scan_args(argc, argv, "23", &service, &printer, &job_attributes,
                 &job_ticket, &rb_files);

    ARY2STRARY(rb_files, files);
    rb_papi_status_check(papiJobSubmitByReference(RVAL2SERVICE(service),
                                                  StringValueCStr(printer),
                                                  RVAL2ATTRLST(job_attributes),
                                                  RVAL2JT(job_ticket),
                                                  files,
                                                  &job));
    rb_job = JOB2RVAL(job);
    rb_ivar_set(rb_job, id_service, service);
    return rb_job;
}

static VALUE
rb_papi_job_validate(int argc, VALUE *argv, VALUE self)
{
    VALUE service, printer, job_attributes, job_ticket, rb_files;
    VALUE rb_job;
    papi_job_t job;
    char **files;

    rb_scan_args(argc, argv, "23", &service, &printer, &job_attributes,
                 &job_ticket, &rb_files);

    ARY2STRARY(rb_files, files);
    rb_papi_status_check(papiJobValidate(RVAL2SERVICE(service),
                                         StringValueCStr(printer),
                                         RVAL2ATTRLST(job_attributes),
                                         RVAL2JT(job_ticket),
                                         files,
                                         &job));
    rb_job = JOB2RVAL(job);
    rb_ivar_set(rb_job, id_service, service);
    return rb_job;
}


static VALUE
rb_papi_job_query(int argc, VALUE *argv, VALUE self)
{
    VALUE service, printer, job_id, rb_requested_attrs;
    VALUE rb_job;
    char **requested_attrs;
    papi_job_t job;

    rb_scan_args(argc, argv, "31", &service, &printer, &job_id,
                 &rb_requested_attrs);

    ARY2STRARY(rb_requested_attrs, requested_attrs);
    rb_papi_status_check(papiJobQuery(RVAL2SERVICE(service),
                                      StringValueCStr(printer),
                                      NUM2INT(job_id),
                                      requested_attrs,
                                      &job));

    rb_job = JOB2RVAL(job);
    rb_ivar_set(rb_job, id_service, service);
    return rb_job;
}

static VALUE
rb_papi_job_modify(VALUE self, VALUE attributes)
{
    VALUE rb_job, service;
    papi_job_t job, new_job;

    job = RVAL2JOB(self);
    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobModify(RVAL2SERVICE(service),
                                       papiJobGetPrinterName(job),
                                       papiJobGetId(job),
                                       RVAL2ATTRLST(attributes),
                                       &new_job));

    rb_job = JOB2RVAL(job);
    rb_ivar_set(rb_job, id_service, service);
    return rb_job;
}

static VALUE
rb_papi_job_move(VALUE self, VALUE destination)
{
    VALUE service;
    papi_job_t job;

    job = RVAL2JOB(self);
    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobMove(RVAL2SERVICE(service),
                                     papiJobGetPrinterName(job),
                                     papiJobGetId(job),
                                     StringValueCStr(destination)));
    return Qnil;
}

static VALUE
rb_papi_job_cancel(VALUE self)
{
    VALUE service;
    papi_job_t job;

    job = RVAL2JOB(self);
    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobCancel(RVAL2SERVICE(service),
                                       papiJobGetPrinterName(job),
                                       papiJobGetId(job)));
    return Qnil;
}

static VALUE
rb_papi_job_hold(VALUE self)
{
    VALUE service;
    papi_job_t job;

    job = RVAL2JOB(self);
    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobHold(RVAL2SERVICE(service),
                                     papiJobGetPrinterName(job),
                                     papiJobGetId(job)));
    return Qnil;
}

static VALUE
rb_papi_job_release(VALUE self)
{
    VALUE service;
    papi_job_t job;

    job = RVAL2JOB(self);
    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobRelease(RVAL2SERVICE(service),
                                        papiJobGetPrinterName(job),
                                        papiJobGetId(job)));
    return Qnil;
}

static VALUE
rb_papi_job_restart(VALUE self)
{
    VALUE service;
    papi_job_t job;

    job = RVAL2JOB(self);
    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobRestart(RVAL2SERVICE(service),
                                        papiJobGetPrinterName(job),
                                        papiJobGetId(job)));
    return Qnil;
}

static VALUE
rb_papi_job_promote(VALUE self)
{
    VALUE service;
    papi_job_t job;

    job = RVAL2JOB(self);
    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobPromote(RVAL2SERVICE(service),
                                        papiJobGetPrinterName(job),
                                        papiJobGetId(job)));
    return Qnil;
}


/* Job accessors */
static VALUE
rb_papi_job_get_attribute_list(VALUE self)
{
    return ATTRLST2RVAL_NO_FREE(papiJobGetAttributeList(RVAL2JOB(self)));
}

static VALUE
rb_papi_job_get_printer_name(VALUE self)
{
    return rb_str_new2(papiJobGetPrinterName(RVAL2JOB(self)));
}

static VALUE
rb_papi_job_get_id(VALUE self)
{
    return INT2NUM(papiJobGetId(RVAL2JOB(self)));
}

static VALUE
rb_papi_job_get_job_ticket(VALUE self)
{
    return JT2RVAL_NO_FREE(papiJobGetJobTicket(RVAL2JOB(self)));
}


/* non-standard API */
static VALUE
rb_papi_job_get_name(VALUE self)
{
    char *name;
    rb_papi_status_check(papiJobGetName(RVAL2JOB(self), &name));
    return rb_str_new2(name);
}

static VALUE
rb_papi_job_get_owner(VALUE self)
{
    char *owner;
    rb_papi_status_check(papiJobGetOwner(RVAL2JOB(self), &owner));
    return rb_str_new2(owner);
}

static VALUE
rb_papi_job_get_size_in_kilobytes(VALUE self)
{
    int size;
    rb_papi_status_check(papiJobGetSizeInKilobytes(RVAL2JOB(self), &size));
    return INT2NUM(size);
}

static VALUE
rb_papi_job_get_state(VALUE self)
{
    int state;
    rb_papi_status_check(papiJobGetState(RVAL2JOB(self), &state));
    return INT2NUM(state);
}

static VALUE
rb_papi_job_get_state_reasons(VALUE self)
{
    char *reasons;
    rb_papi_status_check(papiJobGetStateReasons(RVAL2JOB(self), &reasons));
    return rb_str_new2(reasons);
}

static VALUE
rb_papi_job_get_state_message(VALUE self)
{
    char *message;
    rb_papi_status_check(papiJobGetStateMessage(RVAL2JOB(self), &message));
    return rb_str_new2(message);
}


/* JobStream related */
static VALUE
rb_papi_job_stream_callback(VALUE info)
{
    VALUE callback;

    callback = rb_ary_shift(info);
    return rb_apply(callback, id_call, info);
}

static VALUE
rb_papi_job_stream_close(VALUE self)
{
    VALUE service;
    papi_job_t job;

    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobStreamClose(RVAL2SERVICE(service),
                                            RVAL2JS(self),
                                            &job));

    return JOB2RVAL(job);
}

static VALUE
rb_papi_job_stream_open(int argc, VALUE *argv, VALUE self)
{
    VALUE service, printer, job_attributes, job_ticket;
    VALUE rb_stream, result;
    papi_stream_t stream;

    rb_scan_args(argc, argv, "22", &service, &printer,
                 &job_attributes, &job_ticket);
    rb_papi_status_check(papiJobStreamOpen(RVAL2SERVICE(service),
                                           StringValueCStr(printer),
                                           RVAL2ATTRLST(job_attributes),
                                           RVAL2JT(job_ticket),
                                           &stream));

    rb_stream = JS2RVAL_NO_FREE(stream);
    rb_ivar_set(rb_stream, id_service, service);

    if (rb_block_given_p()) {
        result = rb_ensure(rb_papi_job_stream_callback,
                           rb_ary_new3(2, rb_block_proc(), rb_stream),
                           rb_papi_job_stream_close, rb_stream);
    } else {
        result = rb_stream;
    }

    return result;
}

static VALUE
rb_papi_job_stream_write(VALUE self, VALUE buffer)
{
    VALUE service;

    service = rb_ivar_get(self, id_service);
    rb_papi_status_check(papiJobStreamWrite(RVAL2SERVICE(service),
                                            RVAL2JS(self),
                                            StringValueCStr(buffer),
                                            RSTRING(buffer)->len));
    return Qnil;
}



void
Init_rb_papi_job(VALUE mPAPI)
{
    VALUE mListJobs, mJobTicketFormat;

    id_call = rb_intern("call");
    id_service = rb_intern("service");

    mListJobs = rb_define_module_under(mPAPI, "ListJobs");
    mJobTicketFormat = rb_define_module_under(mPAPI, "JobTicketFormat");

    cJob = rb_define_class_under(mPAPI, "Job", rb_cData);
    cJobTicket = rb_define_class_under(mPAPI, "JobTicket", rb_cData);
    cJobStream = rb_define_class_under(mPAPI, "JobStream", rb_cData);

    /* PAPI_LIST_JOBS_* */
    rb_define_const(mListJobs, "OTHERS", INT2NUM(PAPI_LIST_JOBS_OTHERS));
    rb_define_const(mListJobs, "COMPLETED", INT2NUM(PAPI_LIST_JOBS_COMPLETED));
    rb_define_const(mListJobs, "NOT_COMPLETED",
                    INT2NUM(PAPI_LIST_JOBS_NOT_COMPLETED));
    rb_define_const(mListJobs, "ALL", INT2NUM(PAPI_LIST_JOBS_ALL));

    /* PAPI_JT_FORMAT_* */
    rb_define_const(mJobTicketFormat, "JDF", INT2NUM(PAPI_JT_FORMAT_JDF));
    rb_define_const(mJobTicketFormat, "PWG", INT2NUM(PAPI_JT_FORMAT_PWG));

    /* JobTicket related */
    rb_define_alloc_func(cJobTicket, rb_papi_job_ticket_allocate);
    rb_define_method(cJobTicket, "initialize",
                     rb_papi_job_ticket_initialize, 3);

    rb_define_method(cJobTicket, "format", rb_papi_job_ticket_get_format, 0);
    rb_define_method(cJobTicket, "data", rb_papi_job_ticket_get_data, 0);
    rb_define_method(cJobTicket, "file_name",
                     rb_papi_job_ticket_get_file_name, 0);

    rb_define_method(cJobTicket, "format=", rb_papi_job_ticket_set_format, 1);
    rb_define_method(cJobTicket, "data=", rb_papi_job_ticket_set_data, 1);
    rb_define_method(cJobTicket, "file_name=",
                     rb_papi_job_ticket_set_file_name, 1);

    /* Job related */
    rb_define_singleton_method(cJob, "submit", rb_papi_job_submit, -1);
    rb_define_singleton_method(cJob, "submit_by_reference",
                               rb_papi_job_submit_by_reference, -1);
    rb_define_singleton_method(cJob, "validate", rb_papi_job_validate, -1);

    rb_define_method(cJob, "query", rb_papi_job_query, -1);
    rb_define_method(cJob, "modify", rb_papi_job_modify, 1);
    rb_define_method(cJob, "move", rb_papi_job_move, 1);
    rb_define_method(cJob, "cancel", rb_papi_job_cancel, 0);
    rb_define_method(cJob, "hold", rb_papi_job_hold, 0);
    rb_define_method(cJob, "release", rb_papi_job_release, 0);
    rb_define_method(cJob, "restart", rb_papi_job_restart, 0);
    rb_define_method(cJob, "promote", rb_papi_job_promote, 0);

    /* non-standard API */
    rb_define_method(cJob, "name", rb_papi_job_get_name, 0);
    rb_define_method(cJob, "owner", rb_papi_job_get_owner, 0);
    rb_define_method(cJob, "size_in_kilobytes",
                     rb_papi_job_get_size_in_kilobytes, 0);
    rb_define_method(cJob, "state", rb_papi_job_get_state, 0);
    rb_define_method(cJob, "state_reasons", rb_papi_job_get_state_reasons, 0);
    rb_define_method(cJob, "state_message", rb_papi_job_get_state_message, 0);

    /* Job accessors */
    rb_define_method(cJob, "attributes", rb_papi_job_get_attribute_list, 0);
    rb_define_method(cJob, "printer_name", rb_papi_job_get_printer_name, 0);
    rb_define_method(cJob, "id", rb_papi_job_get_id, 0);
    rb_define_method(cJob, "ticket", rb_papi_job_get_job_ticket, 0);

    /* JobStream related */
    rb_define_singleton_method(cJobStream, "open",
                               rb_papi_job_stream_open, -1);
    rb_define_method(cJobStream, "write", rb_papi_job_stream_write, 1);
    rb_define_method(cJobStream, "close", rb_papi_job_stream_close, 0);
}
