/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */

#include "rb-papi.h"

static VALUE cMetadata;
static ID id_new, id_to_s, id_name, id_value, id_metadata;

papi_metadata_t
rb_papi_r2c_metadata(VALUE data)
{
    return NUM2INT(rb_ivar_get(data, id_value));
}

VALUE
rb_papi_c2r_metadata(papi_metadata_t data)
{
    return rb_hash_aref(rb_ivar_get(cMetadata, id_metadata), INT2NUM(data));
}

static VALUE
rb_papi_metadata_initialize(VALUE self, VALUE name, VALUE value)
{
    rb_ivar_set(self, id_name, name);
    rb_ivar_set(self, id_value, value);
    return Qnil;
}

static VALUE
rb_papi_metadata_to_s(VALUE self)
{
    VALUE result;

    result = rb_str_new2("#<");
    rb_str_concat(result, rb_funcall(rb_obj_class(self), id_name, 0));
    rb_str_cat2(result, ":");
    rb_str_concat(result, rb_ivar_get(self, id_name));
    rb_str_cat2(result, ">");

    return result;
}

#define DEF_METADATA(klass, name)                               \
    do {                                                        \
        VALUE metadata = rb_funcall(klass, id_new, 2,           \
                                    rb_str_new2(# name),        \
                                    INT2NUM(PAPI_ ## name));    \
        rb_define_const(klass, # name, metadata);               \
        rb_hash_aset(rb_ivar_get(klass, id_metadata),           \
                     INT2NUM(PAPI_ ## name), metadata);         \
    } while (0)

void
Init_rb_papi_metadata(VALUE mPAPI)
{
    id_new = rb_intern("new");
    id_to_s = rb_intern("to_s");
    id_name = rb_intern("name");
    id_value = rb_intern("@value");
    id_metadata = rb_intern("metadata");

    cMetadata = rb_define_class_under(mPAPI, "Metadata", rb_cObject);
    rb_ivar_set(cMetadata, id_metadata, rb_hash_new());

    rb_attr(cMetadata, rb_intern("value"), TRUE, FALSE, FALSE);
    rb_define_method(cMetadata, "initialize", rb_papi_metadata_initialize, 2);
    rb_define_method(cMetadata, "to_s", rb_papi_metadata_to_s, 0);

    DEF_METADATA(cMetadata, UNSUPPORTED);
    DEF_METADATA(cMetadata, DEFAULT);
    DEF_METADATA(cMetadata, UNKNOWN);
    DEF_METADATA(cMetadata, NO_VALUE);
    DEF_METADATA(cMetadata, NOT_SETTABLE);
    DEF_METADATA(cMetadata, DELETE);
}
