/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */

#include "rb-papi.h"

static VALUE cPrinter;

static void
no_free(void *ptr)
{
}

VALUE
rb_papi_c2r_printer(papi_printer_t printer)
{
    return Data_Wrap_Struct(cPrinter, 0, papiPrinterFree, printer);
}

VALUE
rb_papi_c2r_printer_no_free(papi_printer_t printer)
{
    return Data_Wrap_Struct(cPrinter, 0, no_free, printer);
}

static VALUE
rb_papi_printers_list(int argc, VALUE *argv, VALUE self)
{
    VALUE requested_attrs, filter;
    VALUE rb_printers = rb_ary_new();
    papi_printer_t *printers = NULL;
    papi_printer_t *printer = NULL;
    papi_status_t status;

    rb_scan_args(argc, argv, "02", &requested_attrs, &filter);

    status =
        papiPrintersList(RVAL2SERVICE(self),
                         NIL_P(requested_attrs) ? NULL : NULL,
                         NIL_P(filter) ? NULL : NULL,
                         &printers);
    rb_papi_status_check(status);

    if (printers) {
        for (printer = printers; *printer; printer++) {
            rb_ary_push(rb_printers, PRINTER2RVAL_NO_FREE(*printer));
            printer++;
        }
    }

    return rb_printers;
}

static VALUE
rb_papi_printer_query(int argc, VALUE *argv, VALUE self)
{
    VALUE name, rb_requested_attrs, job_attributes;
    char **requested_attrs = NULL;
    papi_printer_t printer;

    rb_scan_args(argc, argv, "12", &name, &rb_requested_attrs, &job_attributes);

    ARY2STRARY(rb_requested_attrs, requested_attrs);

    rb_papi_status_check(papiPrinterQuery(RVAL2SERVICE(self),
                                          StringValueCStr(name),
                                          requested_attrs,
                                          NIL_P(job_attributes) ?
                                            NULL :
                                            RVAL2ATTRLST(job_attributes),
                                          &printer));

    return PRINTER2RVAL(printer);
}

static VALUE
rb_papi_printer_add(VALUE self, VALUE name, VALUE attrs)
{
    papi_printer_t printer;

    rb_papi_status_check(papiPrinterAdd(RVAL2SERVICE(self),
                                        StringValueCStr(name),
                                        RVAL2ATTRLST(attrs),
                                        &printer));

    return PRINTER2RVAL(printer);
}

static VALUE
rb_papi_printer_modify(VALUE self, VALUE name, VALUE attrs)
{
    papi_printer_t printer;

    rb_papi_status_check(papiPrinterModify(RVAL2SERVICE(self),
                                           StringValueCStr(name),
                                           RVAL2ATTRLST(attrs),
                                           &printer));

    return PRINTER2RVAL(printer);
}

static VALUE
rb_papi_printer_remove(VALUE self, VALUE name)
{
    rb_papi_status_check(papiPrinterRemove(RVAL2SERVICE(self),
                                           StringValueCStr(name)));
    return Qnil;
}

static VALUE
rb_papi_printer_disable(int argc, VALUE *argv, VALUE self)
{
    VALUE name, message;
    rb_scan_args(argc, argv, "11", &name, &message);

    rb_papi_status_check(papiPrinterDisable(RVAL2SERVICE(self),
                                            StringValueCStr(name),
                                            NIL_P(message) ?
                                              NULL :
                                              StringValueCStr(message)));
    return Qnil;
}

static VALUE
rb_papi_printer_enable(VALUE self, VALUE name)
{
    rb_papi_status_check(papiPrinterEnable(RVAL2SERVICE(self),
                                           StringValueCStr(name)));
    return Qnil;
}

static VALUE
rb_papi_printer_pause(int argc, VALUE *argv, VALUE self)
{
    VALUE name, message;
    rb_scan_args(argc, argv, "11", &name, &message);

    rb_papi_status_check(papiPrinterPause(RVAL2SERVICE(self),
                                          StringValueCStr(name),
                                          NIL_P(message) ?
                                            NULL :
                                            StringValueCStr(message)));
    return Qnil;
}

static VALUE
rb_papi_printer_resume(VALUE self, VALUE name)
{
    rb_papi_status_check(papiPrinterResume(RVAL2SERVICE(self),
                                           StringValueCStr(name)));
    return Qnil;
}

static VALUE
rb_papi_printer_purge_jobs(VALUE self, VALUE name)
{
    VALUE rb_jobs = rb_ary_new();
    papi_job_t **jobs;
    papi_job_t *job = NULL;

    jobs = &job;
    rb_papi_status_check(papiPrinterPurgeJobs(RVAL2SERVICE(self),
                                              StringValueCStr(name),
                                              jobs));

    for (; *jobs; jobs++) {
        rb_ary_push(rb_jobs, JOB2RVAL(*jobs));
    }

    return rb_jobs;
}

static VALUE
rb_papi_printer_list_jobs(int argc, VALUE *argv, VALUE self)
{
    VALUE rb_jobs = rb_ary_new();
    VALUE name, rb_requested_attrs, type_mask, max_num_jobs;
    char **requested_attrs;
    papi_job_t **jobs;
    papi_job_t *job = NULL;

    rb_scan_args(argc, argv, "13",
                 &name, &rb_requested_attrs, &type_mask, &max_num_jobs);

    ARY2STRARY(rb_requested_attrs, requested_attrs);
    jobs = &job;
    rb_papi_status_check(papiPrinterListJobs(RVAL2SERVICE(self),
                                             StringValueCStr(name),
                                             requested_attrs,
                                             NIL_P(type_mask) ?
                                               PAPI_LIST_JOBS_NOT_COMPLETED :
                                               NUM2INT(type_mask),
                                             NIL_P(max_num_jobs) ?
                                               0 :
                                               NUM2INT(max_num_jobs),
                                             jobs));

    for (; *jobs; jobs++) {
        rb_ary_push(rb_jobs, JOB2RVAL(*jobs));
    }

    return rb_jobs;
}

static VALUE
rb_papi_printer_attribute_list(VALUE self)
{
    papi_attribute_t **attrs = papiPrinterGetAttributeList(RVAL2PRINTER(self));
    return ATTRLST2RVAL_NO_FREE(attrs);
}



/* no-standard API */
static VALUE
rb_papi_printer_get_name(VALUE self)
{
    char *name;
    rb_papi_status_check(papiPrinterGetName(RVAL2PRINTER(self), &name));
    return rb_str_new2(name);
}

static VALUE
rb_papi_printer_get_uri(VALUE self)
{
    char *uri;
    rb_papi_status_check(papiPrinterGetURI(RVAL2PRINTER(self), &uri));
    return rb_str_new2(uri);
}

static VALUE
rb_papi_printer_get_state(VALUE self)
{
    int state;
    rb_papi_status_check(papiPrinterGetState(RVAL2PRINTER(self), &state));
    return INT2NUM(state);
}

static VALUE
rb_papi_printer_get_state_reasons(VALUE self)
{
    char *reasons;
    rb_papi_status_check(papiPrinterGetStateReasons(RVAL2PRINTER(self),
                                                    &reasons));
    return rb_str_new2(reasons);
}

static VALUE
rb_papi_printer_get_state_message(VALUE self)
{
    char *message;
    rb_papi_status_check(papiPrinterGetStateMessage(RVAL2PRINTER(self),
                                                    &message));
    return rb_str_new2(message);
}

static VALUE
rb_papi_printer_is_accepting_jobs(VALUE self)
{
    char boolean;
    rb_papi_status_check(papiPrinterIsAcceptingJobs(RVAL2PRINTER(self),
                                                    &boolean));
    return boolean ? Qtrue : Qfalse;
}

static VALUE
rb_papi_printer_is_enabled(VALUE self)
{
    char boolean;
    rb_papi_status_check(papiPrinterIsEnabled(RVAL2PRINTER(self), &boolean));
    return boolean ? Qtrue : Qfalse;
}



void
Init_rb_papi_printer(VALUE mPAPI)
{
    VALUE cService, mPrinterState;

    cService = rb_const_get(mPAPI, rb_intern("Service"));
    cPrinter = rb_define_class_under(mPAPI, "Printer", rb_cData);

    rb_define_method(cService, "printers", rb_papi_printers_list, -1);
    rb_define_method(cService, "printer_query", rb_papi_printer_query, -1);
    rb_define_method(cService, "printer_add", rb_papi_printer_add, 2);
    rb_define_method(cService, "printer_modify", rb_papi_printer_modify, 2);
    rb_define_method(cService, "printer_remove", rb_papi_printer_remove, 1);
    rb_define_method(cService, "printer_disable", rb_papi_printer_disable, -1);
    rb_define_method(cService, "printer_enable", rb_papi_printer_enable, 1);
    rb_define_method(cService, "printer_pause", rb_papi_printer_pause, -1);
    rb_define_method(cService, "printer_resume", rb_papi_printer_resume, 1);
    rb_define_method(cService, "printer_purge_jobs",
                     rb_papi_printer_purge_jobs, 1);
    rb_define_method(cService, "printer_jobs", rb_papi_printer_list_jobs, -1);

    rb_define_method(cPrinter, "attributes", rb_papi_printer_attribute_list, 0);

    /* non-standard API */
    mPrinterState = rb_define_module_under(mPAPI, "PrinterState");
    rb_define_const(mPrinterState, "IDLE", INT2NUM(PAPI_PRINTER_IDLE));
    rb_define_const(mPrinterState, "PROCESSING",
                    INT2NUM(PAPI_PRINTER_PROCESSING));
    rb_define_const(mPrinterState, "STOPPED", INT2NUM(PAPI_PRINTER_STOPPED));

    rb_define_method(cPrinter, "name", rb_papi_printer_get_name, 0);
    rb_define_method(cPrinter, "uri", rb_papi_printer_get_uri, 0);
    rb_define_method(cPrinter, "state", rb_papi_printer_get_state, 0);
    rb_define_method(cPrinter, "state_reasons",
                     rb_papi_printer_get_state_reasons, 0);
    rb_define_method(cPrinter, "state_message",
                     rb_papi_printer_get_state_message, 0);
    rb_define_method(cPrinter, "accepting_jobs?",
                     rb_papi_printer_is_accepting_jobs, 0);
    rb_define_method(cPrinter, "enabled?", rb_papi_printer_is_enabled, 0);
}
