/* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */

#include "rb-papi.h"

static ID id_call, id_callback;
static VALUE cService;

static void
no_free(void *ptr)
{
}

VALUE
rb_papi_c2r_service_no_free(papi_service_t service)
{
    return Data_Wrap_Struct(cService, NULL, no_free, service);
}

static VALUE
rb_papi_service_allocate(VALUE klass)
{
    return Data_Wrap_Struct(klass, NULL, papiServiceDestroy, NULL);
}

static int
rb_papi_service_auth_cb(papi_service_t service, void *app_data)
{
    VALUE callback = (VALUE)app_data;
    int result = PAPI_FALSE;

    if (!NIL_P(callback)) {
        result = RTEST(rb_funcall(callback, id_call, 1,
                                  SERVICE2RVAL_NO_FREE(service)));
    }

    return result;
}

static VALUE
rb_papi_service_initialize(int argc, VALUE *argv, VALUE self)
{
    VALUE name, user, password, encryption, callback;
    papi_service_t handle;
    papi_status_t status;

    rb_scan_args(argc, argv, "04", &name, &user, &password, &encryption);

    callback = rb_block_given_p() ? rb_block_proc() : Qnil;
    status =
        papiServiceCreate(&handle,
                          NIL_P(name) ? NULL : StringValuePtr(name),
                          NIL_P(user) ? NULL : StringValuePtr(user),
                          NIL_P(password) ? NULL : StringValuePtr(password),
                          rb_papi_service_auth_cb,
                          NIL_P(encryption) ?
                            PAPI_ENCRYPT_IF_REQUESTED :
                            NUM2INT(encryption),
                          (void *)callback);
    rb_papi_status_check(status);

    RDATA(self)->data = handle;
    rb_ivar_set(self, id_callback, callback);

    return Qnil;
}

static VALUE
rb_papi_service_get_service_name(VALUE self)
{
    return rb_str_new2(papiServiceGetServiceName(RVAL2SERVICE(self)));
}

static VALUE
rb_papi_service_get_status_message(VALUE self)
{
    return rb_str_new2(papiServiceGetStatusMessage(RVAL2SERVICE(self)));
}

static VALUE
rb_papi_service_get_attribute_list(VALUE self)
{
    papi_attribute_t **attr;
    attr = papiServiceGetAttributeList(RVAL2SERVICE(self));
    return ATTRLST2RVAL_NO_FREE(attr);
}


static VALUE
rb_papi_service_get_user_name(VALUE self)
{
    return rb_str_new2(papiServiceGetUserName(RVAL2SERVICE(self)));
}

static VALUE
rb_papi_service_get_password(VALUE self)
{
    return rb_str_new2(papiServiceGetPassword(RVAL2SERVICE(self)));
}

static VALUE
rb_papi_service_get_encription(VALUE self)
{
    return INT2NUM(papiServiceGetEncryption(RVAL2SERVICE(self)));
}

static VALUE
rb_papi_service_get_auth_callback(VALUE self)
{
    return rb_ivar_get(self, id_callback);
}


static VALUE
rb_papi_service_set_user_name(VALUE self, VALUE user_name)
{
    rb_papi_status_check(papiServiceSetUserName(RVAL2SERVICE(self),
                                                StringValueCStr(user_name)));
    return Qnil;
}

static VALUE
rb_papi_service_set_password(VALUE self, VALUE password)
{
    rb_papi_status_check(papiServiceSetPassword(RVAL2SERVICE(self),
                                                StringValueCStr(password)));
    return Qnil;
}

static VALUE
rb_papi_service_set_encription(VALUE self, VALUE encription)
{
    rb_papi_status_check(papiServiceSetEncryption(RVAL2SERVICE(self),
                                                  NUM2INT(encription)));
    return Qnil;
}

static VALUE
rb_papi_service_set_auth_callback(VALUE self)
{
    VALUE callback;

    callback = rb_block_proc();
    rb_ivar_set(self, id_callback, callback);
    rb_papi_status_check(papiServiceSetAppData(RVAL2SERVICE(self),
                                               (void *)callback));
    return Qnil;
}

void
Init_rb_papi_service(VALUE mPAPI)
{
    id_call = rb_intern("call");
    id_callback = rb_intern("callback");

    cService = rb_define_class_under(mPAPI, "Service", rb_cData);

    rb_define_alloc_func(cService, rb_papi_service_allocate);
    rb_define_method(cService, "initialize", rb_papi_service_initialize, -1);

    rb_define_method(cService, "name", rb_papi_service_get_service_name, 0);
    rb_define_method(cService, "status_message",
                     rb_papi_service_get_status_message, 0);
    rb_define_method(cService, "attributes",
                     rb_papi_service_get_attribute_list, 0);

    rb_define_method(cService, "user_name", rb_papi_service_get_user_name, 0);
    rb_define_method(cService, "password", rb_papi_service_get_password, 0);
    rb_define_method(cService, "encription",
                     rb_papi_service_get_encription, 0);
    rb_define_method(cService, "auth_callback",
                     rb_papi_service_get_auth_callback, 0);

    rb_define_method(cService, "user_name=", rb_papi_service_set_user_name, 1);
    rb_define_method(cService, "password=", rb_papi_service_set_password, 1);
    rb_define_method(cService, "encription=",
                     rb_papi_service_set_encription, 1);
    rb_define_method(cService, "auth_callback=",
                     rb_papi_service_set_auth_callback, 1);
}
